library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity audio_out is
port (
  CLK : in std_logic;
  RST : in std_logic;
  
  sample_in : in std_logic_vector(15 downto 0);
  
  AU_CSN_N : out std_logic;
  AU_BCLK  : out std_logic;
  AU_MCLK  : out std_logic;
  AU_LRCK  : out std_logic;
  AU_SDTI  : out std_logic
  );
end audio_out;


architecture Behavioral of audio_out is

  signal dir                        : std_logic;
  signal div                        : std_logic_vector(2 downto 0);

  signal sampleClock                : std_logic_vector(3 downto 0);
  signal sample                     : std_logic_vector(15 downto 0);
  signal clkcnt                     : std_logic_vector(30 downto 0);
  signal audiocnt_dac, audiocnt_adc : std_logic_vector(11 downto 0);

  signal audio_clk : std_logic;
  signal lrck : std_logic;

begin
  
  AU_CSN_N <= '1'; -- no chip select as we don't write the ctrl regs

  -- generate the 3 clocks: master, serial, frame
  process(CLK, RST)
  begin
    if rst = '1' then 
      clkcnt <= "0000000000000000000000000000000";
    elsif clk'event and clk='1' then
      clkcnt <= clkcnt + 1;
      
    end if;
  end process;

  AU_MCLK   <= clkcnt(1); -- master clock, 12.5 MHz
  audio_clk <= clkcnt(4); -- this is the serial clock, 1.5625 Mhz
  AU_BCLK   <= not audio_clk; -- dont't ask but read AK4565 specs
  

  -- this process was alreqdy here, so i left it here.
  process(audio_clk, RST)
  begin
    if rst = '1' then 
      audiocnt_dac <= "000000000000";
      audiocnt_adc <= "111111111111";
      lrck <= '0'; 
    elsif audio_clk'event and audio_clk='1' then
      audiocnt_dac <= audiocnt_dac + 1;
      audiocnt_adc <= audiocnt_adc + 1;
      lrck <= not audiocnt_dac(4);
    end if;
  end process;

  AU_LRCK <= lrck; -- audio clock, 48.828 kHz
   
  process(audio_clk, sampleclock)
  begin
    if audio_clk'event and audio_clk='1' then
      AU_SDTI     <= sample(15);
      sample      <= sample(14 downto 0) & '0';
      sampleClock <= sampleClock + 1;
    end if;

    if sampleClock="0000" then
      sample      <= sample_in;
    end if;
  end process;
    
end architecture;


